@@ -17,8 +17,9 @@ from socket import (
     socket,
 )
 from threading import Thread, Timer
-from time import sleep
+from time import monotonic, sleep
 from typing import Dict, Optional, Set
+from uuid import uuid4
 from wsgiref import simple_server
 
 from werkzeug.serving import WSGIRequestHandler
@@ -57,6 +58,8 @@ class Handler(WSGIRequestHandler):
     ):
         self.cb_request_received = cb_request_received
         self.env = None
+        self.uuid = uuid4()
+        self.request_received_at = None
 
         with DeferrableCleanup(cb_processing_done) as c:
             if isinstance(server.app, OutOfBandResponder):
@@ -95,15 +98,23 @@ class Handler(WSGIRequestHandler):
         return success
 
     def log_message(self, format, *args):
-        LOGGER.info(format, *args)
+        LOGGER.info("[%s] " + format, self.uuid, *args)
 
     def log_request(self, code="-", size="-"):
+        self.request_received_at = monotonic()
         if isinstance(code, HTTPStatus):
             code = code.value
-        LOGGER.info('"%s" %s %s', self.requestline, str(code), str(size))
+        LOGGER.info(
+            '[%s] "%s" %s %s', self.uuid, self.requestline, str(code), str(size)
+        )
 
     def log_error(self, format, *args):
-        LOGGER.error(format, *args)
+        LOGGER.error("[%s] " + format, self.uuid, *args)
+
+    def log_elapsed(self):
+        LOGGER.info(
+            "[%s] Elapsed: %s" % (self.uuid, monotonic() - self.request_received_at)
+        )
 
 
 class LoadBalancer:
@@ -128,7 +139,7 @@ class LoadBalancer:
                 "Failed to refresh endpoints for %s:%d", self.host, self.port
             )
         t = Timer(self.REFRESH_INTERVAL, self.refresh)
-        t.setDaemon(True)
+        t.daemon = True
         t.start()
 
 
@@ -146,7 +157,9 @@ class Server:
         app,
         *,
         load_balancer=LoadBalancer,
-        redirect_port: Optional[int] = None
+        redirect_port: Optional[int] = None,
+        oob_cleanup_interval_seconds: Optional[float] = None,
+        oob_item_expiry_seconds: Optional[float] = None,
     ):
         (host, port) = addr
         # Set up base environment
@@ -164,7 +177,14 @@ class Server:
         self.real_app = app
         self.worker_tracker = ThreadsTracker()
         self.app = (
-            OutOfBandResponder(app, redirect_port) if redirect_port is not None else app
+            OutOfBandResponder(
+                app,
+                redirect_port,
+                oob_cleanup_interval_seconds,
+                oob_item_expiry_seconds,
+            )
+            if redirect_port is not None
+            else app
         )
 
     def worker(self):
@@ -173,7 +193,7 @@ class Server:
                 self._fetch_and_run_one_job(addr)
 
     def _fetch_and_run_one_job(
-        self, addr, cb_request_received=None, cb_processing_done=lambda _: None
+        self, addr, cb_request_received=None, cb_processing_done=lambda: None
     ):
         with socket(AF_INET, SOCK_STREAM) as s:
             self.sockets_waiting_for_request.add(s)
@@ -192,7 +212,9 @@ class Server:
                 return
 
             try:
-                Handler(s, addr, self, cb_request_received, cb_processing_done)
+                h = Handler(s, addr, self, cb_request_received, cb_processing_done)
+                if h.request_received_at:
+                    h.log_elapsed()
             except BaseException:
                 LOGGER.exception("In WSGI request handling")
                 sleep(1)
@@ -251,7 +273,7 @@ class Server:
 
         with self.worker_tracker, broker_instance_lease as addr:
             self._fetch_and_run_one_job(
-                addr, request_received, getattr(callbacks, "done", lambda _: None)
+                addr, request_received, getattr(callbacks, "done", lambda: None)
             )
 
     def log(self, type, msg, *args, **kwargs):
